@@ -72,7 +72,9 @@ void EpsonProjector1::populateProtocol(
   addKey("COLOR-MODE", PictureMode_Key, 0x8F, 8);
 
   addKey("POWER", Power_Key, 0x90, 8);
+  addKey("Power on", PowerOn_Key, 0x90, 8);
   addKey("Power on", Green_Key, 0x90, 8); // This is a hack
+  addKey("Power off", PowerOff_Key, 0x91, 8);
   addKey("Power off", Red_Key, 0x91, 8); // also hack
   addKey("PAUSE", Pause_Key, 0x92, 8);
   addKey("MUTE", Mute_Key, 0x93, 8); // "Blank"
