@@ -207,6 +207,12 @@ PIRKeysetManager::PIRKeysetManager()
   setupKeyset(new BekoTV1(++counter));
 
   setupKeyset(new BenQTV1(++counter));
+  setupKeyset(new BenQProjector1(++counter));
+  setupKeyset(new BenQProjector1a(++counter));
+  setupKeyset(new BenQProjector2(++counter));
+  setupKeyset(new BenQProjector3(++counter));
+  setupKeyset(new BenQProjector4(++counter));
+  setupKeyset(new BenQProjector5(++counter));
 
   setupKeyset(new BlaupunktVCR1(++counter));
 
@@ -238,6 +244,7 @@ PIRKeysetManager::PIRKeysetManager()
   setupKeyset(new CiscoSTB1(++counter));
   setupKeyset(new CiscoSTB2(++counter));
   setupKeyset(new CiscoSTB3(++counter));
+  setupKeyset(new CiscoSTB4(++counter));
 
   setupKeyset(new ComproTVCard1(++counter));
 
@@ -533,6 +540,7 @@ PIRKeysetManager::PIRKeysetManager()
   setupKeyset(new MitsubishiVCR1(++counter));
   setupKeyset(new MitsubishiVCR1a(++counter));
   setupKeyset(new MitsubishiProjector1(++counter));
+  setupKeyset(new MitsubishiProjector2(++counter));
 
   setupKeyset(new Motorola4DTV(++counter));
   setupKeyset(new MotorolaSTB1(++counter));
@@ -697,12 +705,9 @@ PIRKeysetManager::PIRKeysetManager()
   setupKeyset(new SamsungTV1(++counter));
   setupKeyset(new SamsungTV1a(++counter));
   setupKeyset(new SamsungTV1b(++counter));
-  setupKeyset(new SamsungTV1c(++counter));
-  setupKeyset(new SamsungTV1d(++counter));
-  setupKeyset(new SamsungTV1e(++counter));
-  setupKeyset(new SamsungTV1f(++counter));
   setupKeyset(new SamsungTV2(++counter));
-  setupKeyset(new SamsungTV2a(++counter));
+  setupKeyset(new SamsungTV3(++counter));
+  setupKeyset(new SamsungTV3a(++counter));
   setupKeyset(new SamsungVCR1(++counter));
   setupKeyset(new SamsungVCR1a(++counter));
   setupKeyset(new SamsungVCR1b(++counter));
