@@ -15,6 +15,8 @@ enum PIRPanelName
   Input_Panel,
   Adjust_Panel,
   AC_Panel,
+  StatefulAC_Panel,
+  ACTimer_Panel,
   Audio_Panel,
   Camera_Panel,
   Roomba_Panel,
